import React, {useState, useEffect} from 'react'
import {useSelector, useDispatch} from 'react-redux'

import ShippingAddress from './ShippingAddress.js'
import PaymentMethod from './PaymentMethod.js'
import ReviewOrder from './ReviewOrder.js'
import SubmitOrder from './SubmitOrder.js'
import './checkout.css'

import {dispatchOrderInfo} from './../../actions/checkout_actions'


export default function Checkout() {
    

    const dispatch = useDispatch()

    ///////////////////////
    // Shipping Info State
    ///////////////////////
    const [firstName, setFirstName] = useState("Guy")
    const [lastName, setLastName] = useState("Fieri")
    const [shippingAddress, setShippingAddress] = useState("1234 Flavortown Dr")
    const [aptSuiteNo, setAptSuiteNo] = useState("Apt 317")
    const [city, setCity] = useState("Santa Rosa")
    const [state, setState] = useState("California")
    const [zipCode, setZipCode] = useState("90210")


    ///////////////////////
    // Billing Info State
    ///////////////////////
    const [cardNumber, setCardNumber] = useState(["123456789"])
    const [cardHolderName, setCardHolderName] = useState(["Guy Fieri"])
    const [expirationDate, setExpirationDate] = useState(["05/20206"])
    const [cvv, setCVV] = useState(["123"])


    ///////////////////////
    // Order Review / Summary State
    ///////////////////////

    const numCartItems = useSelector(state => state.cartProcessing.numItems)
    const itemsTotal = useSelector(state => state.cartProcessing.totalPrice)
    const taxRate = 0.0725
    const shippingHandling = 9.00


    ///////////////////////
    // Submit Button State
    ///////////////////////
    const [submitButtonActive, setSubmitButtonActive] = useState([0])
    const [allFieldsValidated, setAllFieldsValidated] = useState([0])
    const [errorWhileValidating, setErrorWhileValidating] = useState([0])


    ///////////////////////
    // Form Submission State
    ///////////////////////
    const user = useSelector(state => state.user.currentUser)
    const cart = useSelector(state => state.cart)




    // This effect listens for the submit button to be clicked
    // It then checks each of the required fields to make sure they are non null
    useEffect( () => {

        // when submit button clicked after being inactive
        if (submitButtonActive === 1) {
            
            // fields to make sure are non-empty
            let fieldsToCheck = [
                firstName,
                lastName,
                shippingAddress,
                city,
                state,
                zipCode
            ]
    
            // initially validated, gets flipped if required field is empty
            let validated = 1
            for (let i in fieldsToCheck) {
                if (fieldsToCheck[i].length === 0) {
                    console.log("a required Field Value was empty")
                    validated = 0
                    break
                }
            }
    
            // if validated, propogate to component state that all fields were validated
            if (validated === 1) {
                setAllFieldsValidated(1)
            }

            // if invalid, propogate to component state
            else {
                console.log("Failed to validate")
                setErrorWhileValidating(1)
                setSubmitButtonActive(0)
            }
        }

        // when submit button gets reset from active to inactive
        else if (submitButtonActive === 0) {
            ;
        }

    }, [submitButtonActive])



    // This effect looks for the allFieldsValidated signal and fires off the order submission when complete
    useEffect( () => {
        handleSubmit()
        console.log("Successful submission request")
    }, [allFieldsValidated])



    let handleSubmit = () => {
        console.log("Submitting Order!")

        let orderInfo = {
            "user": {
                "userId": user.userId,
                "email": user.email,
                "firstName": user.firstName,
                "lastName": user.lastName,
                "accessToken": user.accessToken
            },
            "address": {
                "state": state,
                "city": city,
                "zip": zipCode,
                "street": shippingAddress
            },
            "cart": {
                "id": "something",
                "userId": user.userId,
                "cartItems": cart
            }   
        }

        console.log(orderInfo)
        dispatch(dispatchOrderInfo(orderInfo))

    }

    return (
        <div id="checkout-container">

            {/* Collects User's shipping info */}
            <ShippingAddress 
                captureFirstName={setFirstName}
                captureLastName={setLastName}
                captureShippingAddress={setShippingAddress}
                captureAptSuiteNo={setAptSuiteNo}
                captureCity={setCity}
                captureState={setState}
                captureZipCode={setZipCode}
            />

            {/* Collects User's payment info */}
            <PaymentMethod
                cardNumber={cardNumber}
                cardHolderName={cardHolderName}
                expirationDate={expirationDate}
                cvv={cvv} 
                captureCardNumber={setCardNumber}
                captureCardHolderName={setCardHolderName}
                captureExpirationDate={setExpirationDate}
                captureCVV={setCVV}
            />

            {/* Displays info about order. Calculates price with tax / shipping applied */}
            <ReviewOrder
                numCartItems={numCartItems}
                itemsTotal={itemsTotal}
                shippingHandling={shippingHandling}
                taxRate={taxRate}
            />

            {/* Request to submit happens here, initiates input validation and sends if success */}
            <SubmitOrder
                submitButtonActive={submitButtonActive}
                setSubmitButtonActive={setSubmitButtonActive}
                errorWhileValidating={errorWhileValidating}
                allFieldsValidated={allFieldsValidated}
            />
            
        </div>
    )
}


